from typing import Mapping, Optional

from vital.api.api import API


class Link(API):
    """Endpoints for managing link tokens."""

    def create(
        self,
        user_id: str,
        provider: Optional[str] = None,
        redirect_url: Optional[str] = None,
    ) -> Mapping[str, str]:
        """
        Create a Link token.
        :param str user_id: user's id returned by service.
        """
        return self.client.post(
            "/link/token",
            {"user_id": user_id, "provider": provider, "redirect_url": redirect_url},
        )

    def password_provider(
        self, link_token: str, provider: str, username: str, password: str
    ) -> Mapping[str, str]:
        """
        Connect a password auth provider.
        :param str link_token: link_token created.
        :param str provider: Provider name.
        :param str username: username.
        :param str password: password.
        """
        return self.client.post(
            f"/link/provider/password/{provider}",
            {"username": username, "password": password},
            headers={"LinkToken": link_token},
        )

    def email_provider(
        self, link_token: str, provider: str, email: str, region: Optional[str] = None
    ) -> Mapping[str, str]:
        """
        Connect a password auth provider.
        :param str link_token: link_token created.
        :param str provider: Provider name.
        :param str username: username.
        :param str password: password.
        """
        return self.client.post(
            f"/link/provider/email/{provider}",
            {"email": email, "region": region},
            headers={"LinkToken": link_token},
        )

    def oauth_provider(
        self,
        link_token: str,
        provider: str,
    ) -> Mapping[str, str]:
        """
        Get link to oAuth provider.
        :param str link_token: link_token created.
        :param str provider: Provider name.
        """
        return self.client.get(
            f"/link/provider/oauth/{provider}",
            headers={"LinkToken": link_token},
        )



from typing import List, Mapping, Optional

from vital.api.api import API


class Sleep(API):
    """Endpoints for getting sleep data."""

    def get(
        self,
        user_id: str,
        start_date: str,
        end_date: Optional[str] = "",
        provider: Optional[str] = "",
    ) -> Mapping[str, List[Mapping]]:
        """
        GET Sleep data.
        """
        return self.client.get(
            f"/summary/sleep/{user_id}",
            params={
                "start_date": start_date,
                "end_date": end_date,
                "provider": provider,
            },
        )

    def get_stream_for_date_range(
        self,
        user_id: str,
        start_date: str,
        end_date: Optional[str] = "",
        provider: Optional[str] = "",
    ) -> Mapping[str, List[Mapping]]:
        """
        GET Sleep stream data for a single sleep event's within date range.
        """
        return self.client.get(
            f"/summary/sleep/{user_id}/stream",
            params={
                "start_date": start_date,
                "end_date": end_date,
                "provider": provider,
            },
        )

    def get_raw(
        self,
        user_id: str,
        start_date: str,
        end_date: Optional[str] = "",
        provider: Optional[str] = "",
    ) -> Mapping[str, List[Mapping]]:
        """
        GET Sleep data.
        """
        return self.client.get(
            f"/summary/sleep/{user_id}/raw",
            params={
                "start_date": start_date,
                "end_date": end_date,
                "provider": provider,
            },
        )

    def get_stream(self, sleep_id: str) -> Mapping[str, List[Mapping]]:
        """
        GET Sleep stream data for a single sleep event.
        """
        return self.client.get(f"/timeseries/sleep/{sleep_id}/stream")



from typing import List, Mapping, Optional

from vital.api.api import API


class Activity(API):
    """Endpoints for getting activity data."""

    def get(
        self,
        user_id: str,
        start_date: str,
        end_date: Optional[str] = "",
        provider: Optional[str] = "",
    ) -> Mapping[str, List[Mapping]]:
        """
        Get Activity API
        :param str user_id: user's id
        :param str start_date: date in ISO format
        :param Optional[str] end_date: date in ISO format
        :param Optional[str] provider: Provider of data strava etc.
        """
        return self.client.get(
            f"/summary/activity/{user_id}",
            params={
                "start_date": start_date,
                "end_date": end_date,
                "provider": provider,
            },
        )

    def get_raw(
        self,
        user_id: str,
        start_date: str,
        end_date: Optional[str],
        provider: Optional[str] = "",
    ) -> Mapping[str, List[Mapping]]:
        """
        Get Activity API
        :param str user_id: user's id
        :param str start_date: date in ISO format
        :param Optional[str] end_date: date in ISO format
        :param Optional[str] provider: Provider of data strava etc.
        """
        return self.client.get(
            f"/summary/activity/{user_id}/raw",
            params={
                "start_date": start_date,
                "end_date": end_date,
                "provider": provider,
            },
        )



from vital.api.activity import Activity
from vital.api.body import Body
from vital.api.devices import Devices
from vital.api.link import Link
from vital.api.profile import Profile
from vital.api.sleep import Sleep
from vital.api.testkits import Testkits
from vital.api.user import User
from vital.api.vitals import Vitals
from vital.api.webhooks import Webhooks
from vital.api.workouts import Workouts

__all__ = [
    "Link",
    "Body",
    "Activity",
    "Sleep",
    "User",
    "Workouts",
    "Webhooks",
    "Vitals",
    "Testkits",
    "Devices",
    "Profile",
]



from typing import List, Mapping, Optional

from vital.api.api import API


class Profile(API):
    """Endpoints for getting profile data."""

    def get(
        self,
        user_id: str,
        provider: Optional[str] = "",
    ) -> Mapping[str, List[Mapping]]:
        """
        GET Profile data.
        """
        return self.client.get(
            f"/summary/profile/{user_id}",
            params={
                "provider": provider,
            },
        )

    def get_raw(
        self,
        user_id: str,
        provider: Optional[str] = "",
    ) -> Mapping[str, List[Mapping]]:
        """
        GET Profile data.
        """
        return self.client.get(
            f"/summary/profile/{user_id}/raw",
            params={
                "provider": provider,
            },
        )



import json
from functools import partial
from json.decoder import JSONDecodeError
from typing import Any, Dict, Mapping, Optional

import requests

from vital.errors import VitalError
from vital.version import __version__

ALLOWED_METHODS = {"post", "get", "delete"}
DEFAULT_TIMEOUT = 600  # 10 minutes


def _requests_http_request(
    url: str,
    method: str,
    data: Optional[Mapping],
    headers: Dict,
    timeout: int = DEFAULT_TIMEOUT,
    params: Optional[Mapping] = {},
    session: Optional[requests.Session] = None,
) -> Any:
    normalized_method = method.lower()
    headers.update({"User-Agent": "Vital Python v{}".format(__version__)})
    if normalized_method in ALLOWED_METHODS:
        return getattr(session if session else requests, normalized_method)(
            url, json=data, headers=headers, timeout=timeout, params=params
        )
    else:
        raise Exception("Invalid request method {}".format(method))


def _http_request(
    url: str,
    method: str,
    data: Optional[Mapping] = None,
    headers: Dict = {},
    params: Dict = {},
    timeout: int = DEFAULT_TIMEOUT,
    is_json: bool = True,
    session: Optional[requests.Session] = None,
) -> Any:
    response = _requests_http_request(
        url, method, data, headers or {}, timeout, params=params, session=session
    )

    if is_json or response.headers["Content-Type"] == "application/json":
        try:
            response_body = json.loads(response.text)
        except JSONDecodeError:
            raise VitalError.from_response(
                {
                    "error_message": response.text,
                    "error_type": "API_ERROR",
                    "error_code": "INTERNAL_SERVER_ERROR",
                    "causes": [],
                },
                400,
            )
        if not response.ok:
            raise VitalError.from_response(
                response_body.get("detail", response_body), response.status_code
            )
        else:
            return response_body
    else:
        return response.content


# helpers to simplify partial function application
post_request = partial(_http_request, method="POST")
get_request = partial(_http_request, method="GET", data=None)
delete_request = partial(_http_request, method="DELETE", data=None)



from typing import List, Mapping, Optional

from vital.api.api import API


class Workouts(API):
    """Endpoints for getting sleep data."""

    def get(
        self,
        user_id: str,
        start_date: str,
        end_date: Optional[str] = "",
        provider: Optional[str] = "",
    ) -> Mapping[str, List[Mapping]]:
        """
        GET Workout data.
        """
        return self.client.get(
            f"/summary/workouts/{user_id}",
            params={
                "start_date": start_date,
                "end_date": end_date,
                "provider": provider,
            },
        )

    def get_raw(
        self,
        user_id: str,
        start_date: str,
        end_date: Optional[str] = "",
        provider: Optional[str] = "",
    ) -> Mapping[str, List[Mapping]]:
        """
        GET Workout data.
        """
        return self.client.get(
            f"/summary/workouts/{user_id}/raw",
            params={
                "start_date": start_date,
                "end_date": end_date,
                "provider": provider,
            },
        )

    def get_stream(
        self,
        workout_id: str,
    ) -> Mapping:
        """
        GET Workout Stream data.
        """
        return self.client.get(f"/timeseries/workouts/{workout_id}/stream")



from typing import Dict

from svix.webhooks import Webhook as SvixWebhook

from vital.api.api import API


class Webhooks(API):
    """Endpoints for getting sleep data."""

    @staticmethod
    def construct_webhook_event(
        payload: str, headers: dict, webhook_secret: str
    ) -> Dict:
        """
        Parses Webhook events & validates request signature.
        :param str payload: Payload as json string.
        :param str headers: Dictionary of headers.
        :param str webhook_secret: Youre client webhook secret.
        """
        wh = SvixWebhook(webhook_secret)
        return wh.verify(payload, headers)



from typing import Dict, List, Mapping, Optional

from vital.api.api import API


class Testkits(API):
    """Endpoints for managing testkits."""

    def order(
        self,
        user_id: str,
        testkit_id: str,
        patient_address: Dict,
        patient_details: Dict,
        skip_address_validation: bool = False,
    ) -> Mapping[str, str]:
        """
        Create a Link token.
        :param str client_user_id: A non phi user_id.
        """

        return self.client.post(
            "/testkit/orders",
            {
                "user_id": user_id,
                "testkit_id": testkit_id,
                "patient_address": patient_address,
                "patient_details": patient_details,
            },
            headers={"skip-address-validation": str(skip_address_validation)},
        )

    def get(self) -> Mapping[str, str]:
        """
        Get all testkits.
        """
        return self.client.get("/testkit/")

    # For backwards compatibility
    def order_status(self, order_id: str) -> Mapping[str, str]:
        """
        Get order status.
        :param str user_id: The order_id.
        """
        return self.client.get(f"/testkit/orders/{order_id}")

    def get_order(self, order_id: str) -> Mapping[str, str]:
        """
        Get order status.
        :param str user_id: The order_id.
        """
        return self.client.get(f"/testkit/orders/{order_id}")

    def get_orders(
        self, start_date: str, end_date: str, status: Optional[List[str]] = None
    ) -> Mapping[str, str]:
        """
        Get all orders.
        """
        return self.client.get(
            "/testkit/orders",
            {"start_date": start_date, "end_date": end_date, "status": status},
        )

    def cancel_order(self, order_id: str) -> Mapping[str, str]:
        """
        Cancel order.
        :param str user_id: The order_id.
        """
        return self.client.post(f"/testkit/orders/{order_id}/cancel")



import requests

from vital.api import (
    Activity,
    Body,
    Devices,
    Link,
    Profile,
    Sleep,
    Testkits,
    User,
    Vitals,
    Webhooks,
    Workouts,
)
from vital.internal.requester import (
    DEFAULT_TIMEOUT,
    delete_request,
    get_request,
    post_request,
)
from vital.internal.token_handler import TokenHandler
from vital.internal.utils import urljoin

base_urls = {
    "eu": {
        "prod": "https://api.eu.tryvital.io",
        "production": "https://api.eu.tryvital.io",
        "dev": "https://api.dev.eu.tryvital.io",
        "sandbox": "https://api.sandbox.eu.tryvital.io",
    },
    "us": {
        "prod": "https://api.tryvital.io",
        "production": "https://api.tryvital.io",
        "dev": "https://api.dev.tryvital.io",
        "sandbox": "https://api.sandbox.tryvital.io",
    },
}


def get_base_url(environment: str, region: str) -> str:
    if environment == "local":
        return "http://localhost:8000"
    try:
        return base_urls[region][environment]
    except KeyError:
        raise Exception("Environment not supported")


class Client:
    """
    Python Vital API client.
    All of the endpoints documented under the ``vital.api``
    module may be called from a ``vital.Client`` instance.
    """

    def __init__(
        self,
        client_id=None,
        secret=None,
        environment=None,
        timeout=DEFAULT_TIMEOUT,
        api_version="v2",
        region="us",
        api_key=None,
        **kwargs,
    ):
        """
        Initialize a client with credentials.
        :param  str     client_id:          Your Vital client ID
        :arg    str     secret:             Your Vital secret
        :arg    str     environment:        One of ``sandbox`` or ``production``.
        :arg    int     timeout:            Timeout for API requests.
        :arg    str     api_key:            Your Vital api key - can be used to
            replace client ID and secret.
        """
        self.client_id = client_id
        self.client_secret = secret
        self.environment = environment
        self.timeout = timeout
        self.api_version = api_version
        self.base_url = get_base_url(environment, region)
        self.api_key = api_key
        self.headers = {
            "Accept-Encoding": "deflate",
        }
        if self.api_key:
            self.headers["x-vital-api-key"] = self.api_key
        else:
            self.token_handler = TokenHandler(
                self.client_id,
                self.client_secret,
                self.environment,
                audience=kwargs.get("audience"),
                domain=kwargs.get("domain"),
            )
            self.headers["Authorization"] = f"Bearer {self.token_handler.access_token}"
        self.session = requests.Session()
        # Mirror the HTTP API hierarchy
        self.Profile = Profile(self)
        self.Link = Link(self)
        self.Body = Body(self)
        self.Activity = Activity(self)
        self.Sleep = Sleep(self)
        self.User = User(self)
        self.Workouts = Workouts(self)
        self.Webhooks = Webhooks(self)
        self.Vitals = Vitals(self)
        self.Testkits = Testkits(self)
        self.Devices = Devices(self)

    def post(
        self, path, data=None, is_json=True, params={}, headers={}, api_version=None
    ):
        """Make a post request."""
        return self._post(
            path, data, is_json, params, self.session, headers, api_version
        )

    def get(self, path, params={}, headers={}, api_version=None):
        """Make a get request."""
        return self._get(path, params, self.session, headers, api_version)

    def delete(self, path, params={}, api_version=None):
        """Make a delete request."""
        return self._delete(path, params, self.session, api_version)

    def post_public(self, path, data, is_json=True, api_version=None):
        """Make a post request requiring no auth."""
        return self._post(path, data, is_json, self.session, api_version)

    def _post(
        self, path, data, is_json, params={}, session=None, headers={}, api_version=None
    ):
        headers = {
            **self.headers,
            **headers,
        }
        return post_request(
            urljoin(
                self.base_url,
                f"{self.api_version if not api_version else api_version}{path}",
            ),
            data=data,
            timeout=self.timeout,
            is_json=is_json,
            headers=headers,
            params=params,
            session=session,
        )

    def _get(self, path, params={}, session=None, headers={}, api_version=None):
        headers = {
            **self.headers,
            **headers,
        }
        return get_request(
            urljoin(
                self.base_url,
                f"{self.api_version if not api_version else api_version}{path}",
            ),
            timeout=self.timeout,
            headers=headers,
            params=params,
            session=session,
        )

    def _delete(self, path, params={}, session=None, api_version=None):
        headers = {
            **self.headers,
        }
        return delete_request(
            urljoin(
                self.base_url,
                f"{self.api_version if not api_version else api_version}{path}",
            ),
            timeout=self.timeout,
            headers=headers,
            params=params,
            session=session,
        )



from typing import List, Mapping, Optional

from vital.api.api import API


class Devices(API):
    """Endpoints for getting devices data."""

    def get_raw(
        self,
        user_id: str,
        provider: Optional[str] = "",
    ) -> Mapping[str, List[Mapping]]:
        """
        Get Devices data
        :param str user_id: Vital user's id
        :param Optional[str] provider: Provider of data (e.g. strava)
        """
        return self.client.get(
            f"/summary/devices/{user_id}/raw",
            params={
                "provider": provider,
            },
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['vital', 'vital.api', 'vital.internal']

package_data = \
{'': ['*']}

install_requires = \
['PyJWT>=2.0.1,<3.0.0',
 'arrow',
 'auth0-python',
 'importlib-metadata>=3.7.3,<4.0.0',
 'requests',
 'svix>=0.41.2,<0.42.0']

setup_kwargs = {
    'name': 'vital',
    'version': '1.1.25',
    'description': '',
    'long_description': "# vital-python\n\nThe official Python Library for [Vital API](https://docs.tryvital.io)\n\n# Install\n\n```\npip install vital\n```\n\n# Installing locally\n\n```\npoetry build --format sdist\ntar -xvf dist/*-`poetry version -s`.tar.gz -O '*/setup.py' > setup.py\n```\n\n## Documentation\n\nPlease refer to the official [Vital docs](https://docs.tryvital.io) provide a full reference on using this library.\n",
    'author': 'maitham',
    'author_email': 'maitham@tryvital.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/adeptlabs/vital-python',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from typing import List, Mapping, Optional

from vital.api.api import API


class Vitals(API):
    """Endpoints for getting activity data."""

    def _timeseries_request(
        self,
        user_id: str,
        start_date: str,
        resource: str,
        end_date: Optional[str] = "",
        provider: Optional[str] = "",
    ) -> List[Mapping]:
        return self.client.get(
            f"/timeseries/{user_id}/{resource}",
            params={
                "start_date": start_date,
                "end_date": end_date,
                "provider": provider,
            },
        )

    def glucose(
        self,
        user_id: str,
        start_date: str,
        end_date: Optional[str] = "",
        provider: Optional[str] = "",
    ) -> List[Mapping]:
        """
        Get glucose value API
        :param str user_id: user's id
        :param str start_date: date in ISO format
        :param Optional[str] end_date: date in ISO format
        :param Optional[str] provider: Provider of data
        """
        return self._timeseries_request(
            user_id, start_date, "glucose", end_date, provider
        )

    def cholesterol(
        self,
        type: str,
        user_id: str,
        start_date: str,
        end_date: Optional[str] = "",
        provider: Optional[str] = "",
    ) -> List[Mapping]:
        """
        Get glucose value API
        :param str user_id: user's id
        :param str start_date: date in ISO format
        :param Optional[str] end_date: date in ISO format
        :param Optional[str] provider: Provider of data
        """
        return self._timeseries_request(
            user_id, start_date, f"cholesterol/{type}", end_date, provider
        )

    def ige(
        self,
        user_id: str,
        start_date: str,
        end_date: Optional[str] = "",
        provider: Optional[str] = "",
    ) -> List[Mapping]:
        """
        Get glucose value API
        :param str user_id: user's id
        :param str start_date: date in ISO format
        :param Optional[str] end_date: date in ISO format
        :param Optional[str] provider: Provider of data
        """
        return self._timeseries_request(user_id, start_date, "ige", end_date, provider)

    def igg(
        self,
        user_id: str,
        start_date: str,
        end_date: Optional[str] = "",
        provider: Optional[str] = "",
    ) -> List[Mapping]:
        """
        Get glucose value API
        :param str user_id: user's id
        :param str start_date: date in ISO format
        :param Optional[str] end_date: date in ISO format
        :param Optional[str] provider: Provider of data
        """
        return self._timeseries_request(user_id, start_date, "igg", end_date, provider)

    def heartrate(
        self,
        user_id: str,
        start_date: str,
        end_date: Optional[str] = "",
        provider: Optional[str] = "",
    ) -> List[Mapping]:
        """
        Get glucose value API
        :param str user_id: user's id
        :param str start_date: date in ISO format
        :param Optional[str] end_date: date in ISO format
        :param Optional[str] provider: Provider of data
        """
        return self._timeseries_request(
            user_id, start_date, "heartrate", end_date, provider
        )



from typing import List, Mapping

from vital.api.api import API

link_token_field_names = [
    "client_user_id",
]


class User(API):
    """Endpoints for managing link tokens."""

    def create(self, client_user_id: str) -> Mapping[str, str]:
        """
        Create a Link token.
        :param str client_user_id: A non phi user_id.
        """

        return self.client.post("/user/key", {"client_user_id": client_user_id})

    def delete(self, user_id: str) -> Mapping[str, str]:
        """
        Delete user and associated data this is irreversible.
        :param str user_id: Provided user_id
        """

        return self.client.delete(f"/user/{user_id}")

    def get_all(self) -> Mapping[str, str]:
        """
        Get all users.
        """
        return self.client.get("/user/")

    def get(self, user_id: str) -> Mapping[str, str]:
        """
        Get user id.
        :param str user_id: The client user id.
        """
        return self.client.get(f"/user/{user_id}")

    def resolve(self, client_user_id: str) -> Mapping[str, str]:
        """
        Get user details from key.
        :param str client_user_id: The client user id.
        """
        return self.client.get(f"/user/key/{client_user_id}")

    def providers(self, user_id: str) -> List[Mapping[str, str]]:
        """
        Get list of providers.
        :param str user_id: User id provided by organisation.
        """

        return self.client.get(f"/user/providers/{user_id}")

    def deregister_provider(
        self, user_id: str, provider: str
    ) -> List[Mapping[str, str]]:
        """
        Deregister provider.
        :param str user_id: The generated user_id
        :param str provider: Provider to deregister
        """

        return self.client.delete(f"/user/{user_id}/{provider}")

    def refresh(
        self,
        user_id: str,
    ) -> Mapping[str, List[Mapping]]:
        """
        Endpoint for refreshing a user. This endpoint is used
        to kick-off an earlier refresh for a user's providers.
        E.g. Refresh a user's Strava data now, instead of waiting
        for Vital's refresh cycle.
        :param str user_id: users id
        """
        return self.client.post(
            f"/user/refresh/{user_id}",
        )
